! establishment routine
! Suppose seed pool >> establishment rate.
!
! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_stomate/lpj_establish.f90,v 1.9 2009/01/06 15:01:25 ssipsl Exp $
! IPSL (2006)
!  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
MODULE lpj_establish

  ! modules used:

  USE ioipsl
  USE stomate_constants
  USE constantes_veg

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC establish,establish_clear

  ! first call
  LOGICAL, SAVE                                              :: firstcall = .TRUE.
CONTAINS


  SUBROUTINE establish_clear
    firstcall = .TRUE.
  END SUBROUTINE establish_clear

  SUBROUTINE establish (npts, dt, PFTpresent, regenerate, &
       neighbours, resolution, need_adjacent, herbivores, &
       precip_annual, gdd0, lm_lastyearmax, &
       cn_ind, lai, avail_tree, avail_grass,  npp_longterm, &
       leaf_age, leaf_frac, &
       ind, biomass, age, everywhere, co2_to_bm,veget_max, woodmass_ind)
    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER(i_std), INTENT(in)                                  :: npts
    ! Time step of vegetation dynamics (days)
    REAL(r_std), INTENT(in)                                     :: dt
    ! Is pft there
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                  :: PFTpresent
    ! Winter sufficiently cold
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: regenerate
    ! indices of the 8 neighbours of each grid point (1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW)
    INTEGER(i_std), DIMENSION(npts,8), INTENT(in)               :: neighbours
    ! resolution at each grid point in m (1=E-W, 2=N-S)
    REAL(r_std), DIMENSION(npts,2), INTENT(in)                  :: resolution
    ! in order for this PFT to be introduced, does it have to be present in an
    !   adjacent grid box?
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                  :: need_adjacent
    ! time constant of probability of a leaf to be eaten by a herbivore (days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: herbivores
    ! annual precipitation (mm/year)
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: precip_annual
    ! growing degree days (C)
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: gdd0
    ! last year's maximum leaf mass, for each PFT (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: lm_lastyearmax
    ! crown area of individuals (m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: cn_ind
    ! leaf area index OF AN INDIVIDUAL PLANT
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: lai
    ! space availability for trees
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: avail_tree
    ! space availability for grasses
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: avail_grass
    ! longterm NPP, for each PFT (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                :: npp_longterm
    ! "maximal" coverage fraction of a PFT (LAI -> infinity) on ground 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: veget_max

    ! 0.2 modified fields

    ! leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_age
    ! fraction of leaves in leaf age class
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_frac
    ! Number of individuals / m2
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: ind
    ! biomass (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(inout)     :: biomass
    ! mean age (years)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: age
    ! is the PFT everywhere in the grid box or very localized (after its introduction)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: everywhere
    ! biomass uptaken (gC/(m**2 of total ground)/day)
    !NV passage 2D
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                 :: co2_to_bm
    ! woodmass of the individual, needed to calculate crownarea in lpj_crownarea
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                 :: woodmass_ind

    ! 0.3 local

    ! time during which a sapling can be entirely eaten by herbivores (d)
    REAL(r_std)                                                 :: tau_eatup 
    ! new fpc ( foliage protected cover: fractional coverage )
    REAL(r_std), DIMENSION(npts,nvm)                           :: fpc_nat
    ! maximum tree establishment rate, based on climate only
    REAL(r_std), DIMENSION(npts)                                :: estab_rate_max_climate_tree
    ! maximum grass establishment rate, based on climate only
    REAL(r_std), DIMENSION(npts)                                :: estab_rate_max_climate_grass
    ! maximum tree establishment rate, based on climate and fpc
    REAL(r_std), DIMENSION(npts)                                :: estab_rate_max_tree
    ! maximum grass establishment rate, based on climate and fpc
    REAL(r_std), DIMENSION(npts)                                :: estab_rate_max_grass
    ! total natural fpc
    REAL(r_std), DIMENSION(npts)                                :: sumfpc
    ! total fraction occupied by natural vegetation
    REAL(r_std), DIMENSION(npts)                                :: fracnat
    ! total woody fpc
    REAL(r_std), DIMENSION(npts)                                :: sumfpc_wood
    ! for trees, measures the total concurrence for available space
    REAL(r_std), DIMENSION(npts)                                :: spacefight_tree
    ! for grasses, measures the total concurrence for available space
    REAL(r_std), DIMENSION(npts)                                :: spacefight_grass
    ! change in number of individuals /m2 per time step (per day in history file)
    REAL(r_std), DIMENSION(npts,nvm)                           :: d_ind
    ! biomass increase (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts)                                :: bm_new
    ! stem diameter (m)
    REAL(r_std), DIMENSION(npts)                                :: dia
    ! temporary variable
    REAL(r_std), DIMENSION(npts)                                :: b1
    ! new sap mass (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts)                                :: sm2
    ! woodmass of an individual
    REAL(r_std), DIMENSION(npts)                                :: woodmass
    ! carbon mass in youngest leaf age class (gC/m**2 PFT)
    REAL(r_std), DIMENSION(npts)                                :: leaf_mass_young
    ! ratio of hw(above) to total hw, sm(above) to total sm
    REAL(r_std), DIMENSION(npts)                                :: sm_at
    ! reduction factor for establishment if many trees or grasses are present
    REAL(r_std), DIMENSION(npts)                                :: factor
    ! Total carbon mass for all pools
    REAL(r_std), DIMENSION(npts)                                :: total_bm_c
    ! Total sappling biomass for all pools
    REAL(r_std), DIMENSION(npts)                                :: total_bm_sapl
    ! from how many sides is the grid box invaded
    INTEGER(i_std)                                              :: nfrontx
    INTEGER(i_std)                                              :: nfronty
    ! daily establishment rate is large compared to present number of individuals
    !LOGICAL, DIMENSION(npts)                                   :: many_new
    ! flow due to new individuals
    !   veget_max after establishment, to get a proper estimate of carbon and nitrogen 
    REAL(r_std), DIMENSION(npts)                                 :: vn
    !   lai on each PFT surface 
    REAL(r_std), DIMENSION(npts)                                 :: lai_ind

    ! indices
    INTEGER(i_std)                                              :: i,j,k,m

    ! =========================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering establish'

    !
    ! 1 messages and initialization
    !
    tau_eatup = one_year/2.

    IF ( firstcall ) THEN

       WRITE(numout,*) 'establish:'

       WRITE(numout,*) '   > time during which a sapling can be entirely eaten by herbivores (d): ', &
            tau_eatup

       firstcall = .FALSE.

    ENDIF


    IF (control%ok_dgvm) THEN
       !
       ! 2 recalculate fpc
       !

       !
       ! 2.1 Only natural part of the grid cell
       !

       fracnat(:) = 1.
       do j = 2,nvm
          IF ( .NOT. natural(j) ) THEN
             fracnat(:) = fracnat(:) - veget_max(:,j)
          ENDIF
       ENDDO

       !
       ! 2.2 total natural fpc on grid
       !
       sumfpc(:) = zero
       DO j = 2,nvm

          IF ( natural(j) ) THEN
             WHERE(fracnat(:).GT.min_stomate)
                WHERE (lai(:,j) == val_exp) 
                   fpc_nat(:,j) = cn_ind(:,j) * ind(:,j) / fracnat(:)
                ELSEWHERE
                   fpc_nat(:,j) = cn_ind(:,j) * ind(:,j) / fracnat(:) & 
                        * ( 1. - exp( - lm_lastyearmax(:,j) * sla(j) * ext_coeff(j) ) )
                ENDWHERE
             ENDWHERE

             WHERE ( PFTpresent(:,j) )
                sumfpc(:) = sumfpc(:) + fpc_nat(:,j)
             ENDWHERE
          ELSE

             fpc_nat(:,j) = 0.0

          ENDIF

       ENDDO

       !
       ! 2.3 total woody fpc on grid and number of regenerative tree pfts
       !

       sumfpc_wood(:) = zero
       spacefight_tree(:) = zero

       DO j = 2,nvm

          IF ( tree(j) .AND. natural(j) ) THEN

             ! total woody fpc

             WHERE ( PFTpresent(:,j) )
                sumfpc_wood(:) = sumfpc_wood(:) + fpc_nat(:,j)
             ENDWHERE

             ! how many trees are competing? Count a PFT fully only if it is present
             !   on the whole grid box.

             WHERE ( PFTpresent(:,j) .AND. ( regenerate(:,j) .GT. regenerate_crit ) )
                spacefight_tree(:) = spacefight_tree(:) + everywhere(:,j)
             ENDWHERE

          ENDIF

       ENDDO

       !
       ! 2.4 number of natural grasses
       !

       spacefight_grass(:) = zero

       DO j = 2,nvm

          IF ( .NOT. tree(j) .AND. natural(j) ) THEN

             ! how many grasses are competing? Count a PFT fully only if it is present
             !   on the whole grid box.

             WHERE ( PFTpresent(:,j) )
                spacefight_grass(:) = spacefight_grass(:) + everywhere(:,j)
             ENDWHERE

          ENDIF

       ENDDO

       !
       ! 3 establishment rate
       !

       !
       ! 3.1 maximum establishment rate, based on climate only
       !

       WHERE ( ( precip_annual(:) .GE. precip_crit ) .AND. ( gdd0(:) .GE. gdd_crit ) )

          estab_rate_max_climate_tree(:) = estab_max_tree
          estab_rate_max_climate_grass(:) = estab_max_grass

       ELSEWHERE

          estab_rate_max_climate_tree(:) = zero
          estab_rate_max_climate_grass(:) = zero

       ENDWHERE

       !
       ! 3.2 reduce maximum tree establishment rate if many trees present.
       !     In the original DGVM, this is done using a step function which yields a
       !     reduction by factor 4 if sumfpc_wood(i) .GT.  fpc_crit - 0.05.
       !     This can lead to small oscillations (without consequences however).
       !     Here, a steady linear transition is used between fpc_crit-0.075 and
       !     fpc_crit-0.025.
       !

       !       factor(:) = 1. - 15. * ( sumfpc_wood(:) - (fpc_crit-.075) )
       !       factor(:) = MAX( 0.25_r_std, MIN( 1._r_std, factor(:) ) )

       !SZ modified according to Smith et al. 2001, 080806
       factor(:)=(1.0-exp(-5.0*(1.0-sumfpc_wood(:))))*(1.0-sumfpc_wood(:))

       estab_rate_max_tree(:) = estab_rate_max_climate_tree(:) * factor(:)

       !
       ! 3.3 Modulate grass establishment rate.
       !     If canopy is not closed (fpc < fpc_crit-0.05), normal establishment.
       !     If canopy is closed, establishment is reduced by a factor 4.
       !     Factor is linear between these two bounds.
       !     This is different from the original DGVM where a step function is
       !     used at fpc_crit-0.05 (This can lead to small oscillations,
       !     without consequences however).
       !

       !       factor(:) = 1. - 15. * ( sumfpc(:) - (fpc_crit-.05) )
       !       factor(:) = MAX( 0.25_r_std, MIN( 1._r_std, factor(:) ) )
       !       estab_rate_max_grass(:) = estab_rate_max_climate_grass(:) * factor(:)

       !SZ modified to true LPJ formulation, grasses are only allowed in the
       !fpc fraction not occupied by trees..., 080806
!NVmodif       estab_rate_max_grass(:)=MAX(0.98-sumfpc(:),zero)
       estab_rate_max_grass(:)=MAX(MIN(estab_rate_max_climate_grass(:),0.98-sumfpc(:)),zero)

       ! SZ: longterm grass NPP for competition between C4 and C3 grasses
       !     to avoid equal veget_max, the idea is that more reestablishment
       !     is possible for the more productive PFT
       factor(:)=min_stomate
       DO j = 2,nvm
          IF ( natural(j) .AND. .NOT.tree(j)) & 
               factor(:)=factor(:)+npp_longterm(:,j) * &
               lm_lastyearmax(:,j) * sla(j)
       ENDDO
       !
       !
       !
       ! 4 do establishment for natural PFTs
       !

       d_ind(:,:) = zero

       DO j = 2,nvm

          ! only for natural PFTs

          IF ( natural(j) ) THEN

             !
             ! 4.1 PFT expansion across the grid box. Not to be confused with areal
             !     coverage.
             !

             IF ( treat_expansion ) THEN

                ! only treat plants that are regenerative and present and still can expand

                DO i = 1, npts

                   IF ( PFTpresent(i,j) .AND. &
                        ( everywhere(i,j) .LT. un ) .AND. &
                        ( regenerate(i,j) .GT. regenerate_crit ) ) THEN

                      ! from how many sides is the grid box invaded (separate x and y directions
                      ! because resolution may be strongly anisotropic)
                      !
                      ! For the moment we only look into 4 direction but that can be extanded (JP) 
                      !
                      nfrontx = 0
                      IF ( neighbours(i,3) .GT. 0 ) THEN
                         IF ( everywhere(neighbours(i,3),j) .GT. 1.-min_stomate ) nfrontx = nfrontx+1
                      ENDIF
                      IF ( neighbours(i,7) .GT. 0 ) THEN
                         IF ( everywhere(neighbours(i,7),j) .GT. 1.-min_stomate ) nfrontx = nfrontx+1
                      ENDIF

                      nfronty = 0
                      IF ( neighbours(i,1) .GT. 0 ) THEN
                         IF ( everywhere(neighbours(i,1),j) .GT. 1.-min_stomate ) nfronty = nfronty+1
                      ENDIF
                      IF ( neighbours(i,5) .GT. 0 ) THEN
                         IF ( everywhere(neighbours(i,5),j) .GT. 1.-min_stomate ) nfronty = nfronty+1
                      ENDIF

                      everywhere(i,j) = &
                           everywhere(i,j) + migrate(j) * dt/one_year * &
                           ( nfrontx / resolution(i,1) + nfronty / resolution(i,2) )

                      IF ( .NOT. need_adjacent(i,j) ) THEN

                         ! in that case, we also assume that the PFT expands from places within
                         ! the grid box (e.g., oasis).

                         everywhere(i,j) = &
                              everywhere(i,j) + migrate(j) * dt/one_year * &
                              2. * SQRT( pi*everywhere(i,j)/(resolution(i,1)*resolution(i,2)) )

                      ENDIF

                      everywhere(i,j) = MIN( everywhere(i,j), 1._r_std )

                   ENDIF

                ENDDO

             ENDIF  ! treat expansion?

             !
             ! 4.2 establishment rate
             !     - Is lower if the PFT is only present in a small part of the grid box
             !       (after its introduction), therefore multiplied by "everywhere".
             !     - Is divided by the number of PFTs that compete ("spacefight").
             !     - Is modulated by space availability (avail_tree, avail_grass).
             !

             IF ( tree(j) ) THEN

                ! 4.2.1 present and regenerative trees

                WHERE ( PFTpresent(:,j) .AND. ( regenerate(:,j) .GT. regenerate_crit ) )


                   d_ind(:,j) = estab_rate_max_tree(:)*everywhere(:,j)/spacefight_tree(:) * &
                        avail_tree(:) * dt/one_year

                ENDWHERE

             ELSE

                ! 4.2.2 present and regenerative grasses

                WHERE ( PFTpresent(:,j) .AND. ( regenerate(:,j) .GT. regenerate_crit )  & 
                     .AND.factor(:).GT.min_stomate .AND. spacefight_grass(:).GT. min_stomate) 

                   d_ind(:,j) = estab_rate_max_grass(:)*everywhere(:,j)/spacefight_grass(:) * &
                        MAX(min_stomate,npp_longterm(:,j)*lm_lastyearmax(:,j)*sla(j)/factor(:)) * fracnat(:) * dt/one_year

                ENDWHERE

             ENDIF  ! tree/grass

          ENDIF ! if natural
       ENDDO ! PFTs

    ELSE ! lpj establishment in static case, SZ 080806, account for real LPJ dynamics in 
       ! prescribed vegetation, i.e. population dynamics within a given area of the 
       ! grid cell

       d_ind(:,:) = 0.0

       DO j = 2,nvm

          ! only for natural PFTs

          WHERE(ind(:,j)*cn_ind(:,j).GT.min_stomate)
             lai_ind(:)=sla(j) * lm_lastyearmax(:,j)/(ind(:,j)*cn_ind(:,j))
          ELSEWHERE
             lai_ind(:)=0.0
          ENDWHERE

          IF ( natural(j) .AND. tree(j)) THEN 

             fpc_nat(:,j) =  MIN(1.0,cn_ind(:,j) * ind(:,j) * & 
                  MAX( ( 1._r_std - exp( - ext_coeff(j) * lai_ind(:) ) ), min_cover ) )
             !fpc_nat(:,j) = max(fpc_nat(:,j),1.-exp(-0.5*sla(j) * lm_lastyearmax(:,j)))


             WHERE (veget_max(:,j).GT.min_stomate.AND.ind(:,j).LE.2.)

                ! only establish into growing stands, ind can become very
                ! large in the static mode because LAI is very low in poor 
                ! growing conditions, favouring continuous establishment. To avoid this
                ! a maximum IND is set. BLARPP: This should be replaced by a 
                ! better stand density criteria
                !
                factor(:)=(1.0-exp(-5.0*(1.0-fpc_nat(:,j))))*(1.0-fpc_nat(:,j))

                estab_rate_max_tree(:) = estab_max_tree * factor(:) 
                !
                ! 4 do establishment for natural PFTs
                !
                d_ind(:,j) = MAX( 0.0, estab_rate_max_tree(:) * dt/one_year)

             ENDWHERE

             !SZ: quickfix: to simulate even aged stand, uncomment the following lines...
             !where (ind(:,j) .LE. min_stomate)
             !d_ind(:,j) = 0.1 !MAX( 0.0, estab_rate_max_tree(:) * dt/one_year)

             WHERE (veget_max(:,j).GT.min_stomate.AND.ind(:,j).EQ.0.0)
                d_ind(:,j) = ind_0*10.
                !          elsewhere
                !d_ind(:,j) =0.0
             endwhere

          ELSEIF ( natural(j) .AND. .NOT.tree(j)) THEN 

             WHERE (veget_max(:,j).GT.min_stomate)

                fpc_nat(:,j) =  cn_ind(:,j) * ind(:,j) * & 
                     MAX( ( 1._r_std - exp( - ext_coeff(j) * lai_ind(:) ) ), min_cover )

                d_ind(:,j) = MAX(0.0 , (1.0-fpc_nat(:,j)) * dt/one_year )

             ENDWHERE

             WHERE (veget_max(:,j).GT.min_stomate.AND.ind(:,j).EQ.0.0)
                d_ind(:,j) = ind_0*10.
             ENDWHERE

          ENDIF

       ENDDO

    ENDIF ! DGVM OR NOT

    DO j = 2,nvm

       ! only for natural PFTs

       IF ( natural(j) ) THEN

          !
          ! 4.3 herbivores reduce establishment rate
          !     We suppose that saplings are vulnerable during a given time after establishment.
          !     This is taken into account by preventively reducing the establishment rate.
          !

          IF ( ok_herbivores ) THEN

             d_ind(:,j) = d_ind(:,j) * EXP( - tau_eatup/herbivores(:,j) )

          ENDIF

          !
          ! 4.4 be sure that ind*cn_ind does not exceed 1
          !SZ This control is now moved to lpj_cover.f90
          !SZ

          !The aim is to control for sum(veget)=1., irrespective of ind*cnd (crowns can overlap as long as
          ! there is enough light
          !
          !SZ: This could be part of the dynamic vegetation problem of Orchidee
          !in conjunction with the wrong formulation of establishment response 
          !to tree fpc above...
          !          WHERE ( ( d_ind(:,j) .GT. zero ) .AND. &
          !                  ( (ind(:,j)+d_ind(:,j))*cn_ind(:,j) .GT. un ) )
          !
          !            d_ind(:,j) = MAX( 1._stnd / cn_ind(:,j) - ind(:,j), zero )
          !
          !          ENDWHERE

          !
          ! 4.5 new properties where there is establishment ( d_ind > 0 )
          !

          ! 4.5.1 biomass.
          !       Add biomass only if d_ind, over one year, is of the order of ind.
          !       (If we don't do this, the biomass density can become very low).
          !       In that case, take biomass from the atmosphere.

          ! compare establishment rate and present number of inidivuals
          !many_new(:) = ( d_ind(:,j) .GT. 0.1 * ind(:,j) )

          ! gives a better vectorization of the VPP

          !IF ( ANY( many_new(:) ) ) THEN

          ! save old leaf mass to calculate leaf age
          leaf_mass_young(:) = leaf_frac(:,j,1) * biomass(:,j,ileaf)
          ! total biomass of existing PFT to limit biomass added from establishment
          total_bm_c(:) = zero

          DO k = 1, nparts
             total_bm_c(:)=total_bm_c(:)+biomass(:,j,k)
          ENDDO
          IF(control%ok_dgvm) THEN
             vn(:)=veget_max(:,j)
          ELSE
             vn(:)=1.0
          ENDIF
          total_bm_sapl(:)=zero
          DO k = 1, nparts
             WHERE(d_ind(:,j).GT.min_stomate.AND.vn(:).GT.min_stomate)

                total_bm_sapl(:) = total_bm_sapl(:) + & 
                     bm_sapl(j,k) * d_ind(:,j) / vn(:)
             ENDWHERE
          ENDDO

          IF(control%ok_dgvm) THEN
             ! SZ calculate new woodmass_ind and veget_max after establishment (needed for correct scaling!)
             ! essential correction for MERGE!
             IF(tree(j))THEN
                DO i=1,npts
                   IF((d_ind(i,j)+ind(i,j)).GT.min_stomate) THEN

                      IF((total_bm_c(i).LE.min_stomate) .OR. (veget_max(i,j) .LE. min_stomate)) THEN

                         ! new wood mass of PFT
                         woodmass_ind(i,j) = &
                              & (((biomass(i,j,isapabove)+biomass(i,j,isapbelow) &
                              & +biomass(i,j,iheartabove)+biomass(i,j,iheartbelow))*veget_max(i,j)) &
                              & +(bm_sapl(j,isapabove)+bm_sapl(j,isapbelow) &
                              & +bm_sapl(j,iheartabove)+bm_sapl(j,iheartbelow))*d_ind(i,j))/(ind(i,j)+d_ind(i,j))

                      ELSE 
                         ! new biomass is added to the labile pool, hence there is no change in CA associated with establishment
                         woodmass_ind(i,j) = &
                              & (biomass(i,j,isapabove)+biomass(i,j,isapbelow) &
                              & +biomass(i,j,iheartabove)+biomass(i,j,iheartbelow))*veget_max(i,j) &
                              & /(ind(i,j)+d_ind(i,j))

                      ENDIF

                      ! new diameter of PFT
                      dia(i) = (woodmass_ind(i,j)/(pipe_density*pi/4.*pipe_tune2)) &
                           &                **(1./(2.+pipe_tune3))
                      vn(i)=(ind(i,j)+d_ind(i,j))*pipe_tune1*MIN(dia(i),maxdia(j))**1.6

                   ENDIF
                ENDDO
             ELSE ! for grasses, cnd=1, so the above calculation cancels
                vn(:)=ind(:,j)+d_ind(:,j)
             ENDIF
          ELSE ! static
             DO i=1,npts
                IF(tree(j).AND.(d_ind(i,j)+ind(i,j)).GT.min_stomate) THEN
                   IF(total_bm_c(i).LE.min_stomate) THEN

                      ! new wood mass of PFT
                      woodmass_ind(i,j) = &
                           & (((biomass(i,j,isapabove)+biomass(i,j,isapbelow) &
                           & +biomass(i,j,iheartabove)+biomass(i,j,iheartbelow))) &
                           & +(bm_sapl(j,isapabove)+bm_sapl(j,isapbelow) &
                           & +bm_sapl(j,iheartabove)+bm_sapl(j,iheartbelow))*d_ind(i,j))/(ind(i,j)+d_ind(i,j))

                   ELSE ! new biomass is added to the labile pool, hence there is no change in CA associated with establishment

                      woodmass_ind(i,j) = &
                           & (biomass(i,j,isapabove)+biomass(i,j,isapbelow) &
                           & +biomass(i,j,iheartabove)+biomass(i,j,iheartbelow)) &
                           & /(ind(i,j)+d_ind(i,j))

                   ENDIF
                ENDIF
             ENDDO

             vn(:)=1.0 ! cannot change in static!, and veget_max implicit in d_ind

          ENDIF
          ! total biomass of PFT added by establishment defined over veget_max ...
          total_bm_sapl(:)=zero
          DO k = 1, nparts
             WHERE(d_ind(:,j).GT.min_stomate.AND.total_bm_c(:).GT.min_stomate.AND.vn(:).GT.min_stomate)

                total_bm_sapl(:) = total_bm_sapl(:) + & 
                     bm_sapl(j,k) * d_ind(:,j) / vn(:)
             ENDWHERE
          ENDDO

          DO k = 1, nparts

             bm_new(:)=zero

             ! first ever establishment, C flows
             WHERE( d_ind(:,j).GT.min_stomate .AND. &
                  total_bm_c(:).LE.min_stomate .AND. &
                  vn(:).GT.min_stomate)
                ! WHERE ( many_new(:) )

                !bm_new(:) = d_ind(:,j) * bm_sapl(j,k) / veget_max (:,j)
                bm_new(:) = d_ind(:,j) * bm_sapl(j,k) / vn(:)

                biomass(:,j,k) = biomass(:,j,k) + bm_new(:)

                co2_to_bm(:,j) = co2_to_bm(:,j) + bm_new(:) / dt

             ENDWHERE

             ! establishment into existing population, C flows
             WHERE(d_ind(:,j).GT.min_stomate.AND.total_bm_c(:).GT.min_stomate)

                bm_new(:) = total_bm_sapl(:) * biomass(:,j,k) / total_bm_c(:)

                biomass(:,j,k) = biomass(:,j,k) + bm_new(:)
                co2_to_bm(:,j) = co2_to_bm(:,j) + bm_new(:) / dt

             ENDWHERE
          ENDDO

          ! reset leaf ages. Should do a real calculation like in the npp routine, 
          ! but this case is rare and not worth messing around.
          ! SZ 080806, added real calculation now, because otherwise leaf_age/leaf_frac
          ! are not initialised for the calculation of vmax, and hence no growth at all.
          ! logic follows that of stomate_npp.f90, just that it's been adjusted for the code here
          !
          ! 4.5.2 Decrease leaf age in youngest class if new leaf biomass is higher than old one.
          !

!!$          WHERE ( many_new(:) )
!!$             leaf_age(:,j,1) = zero
!!$             leaf_frac(:,j,1) = un
!!$          ENDWHERE
!!$
!!$          DO m = 2, nleafages
!!$
!!$             WHERE ( many_new(:) )
!!$                leaf_age(:,j,m) = zero
!!$                leaf_frac(:,j,m) = zero
!!$             ENDWHERE
!!$
!!$          ENDDO

          WHERE ( d_ind(:,j) * bm_sapl(j,ileaf) .GT. min_stomate ) 

             leaf_age(:,j,1) = leaf_age(:,j,1) * leaf_mass_young(:) / &
                  ( leaf_mass_young(:) + d_ind(:,j) * bm_sapl(j,ileaf) )

          ENDWHERE

          leaf_mass_young(:) = leaf_mass_young(:) + d_ind(:,j) * bm_sapl(j,ileaf)

          !
          ! new age class fractions (fraction in youngest class increases)
          !

          ! youngest class: new mass in youngest class divided by total new mass

          WHERE ( biomass(:,j,ileaf) .GT. min_stomate )

             leaf_frac(:,j,1) = leaf_mass_young(:) / biomass(:,j,ileaf)

          ENDWHERE

          ! other classes: old mass in leaf age class divided by new mass

          DO m = 2, nleafages

             WHERE ( biomass(:,j,ileaf) .GT. min_stomate )

                leaf_frac(:,j,m) = leaf_frac(:,j,m) * & 
                     ( biomass(:,j,ileaf) + d_ind(:,j) * bm_sapl(j,ileaf) ) /  biomass(:,j,ileaf)

             ENDWHERE

          ENDDO

          !ENDIF   ! establishment rate is large

          WHERE ( d_ind(:,j) .GT. min_stomate )

             ! 4.5.3 age decreases

             age(:,j) = age(:,j) * ind(:,j) / ( ind(:,j) + d_ind(:,j) )

             ! 4.5.4 new number of individuals

             ind(:,j) = ind(:,j) + d_ind(:,j)

          ENDWHERE

          !
          ! 4.6 eventually convert excess sapwood to heartwood
          !

          !SZ to clarify with Gerhard Krinner: This is theoretically inconsistent because 
          ! the allocation to sapwood and leaves do not follow the LPJ logic in stomate_alloc.f90
          ! hence imposing this here not only solves for the uneveness of age (mixing new and average individual)
          ! but also corrects for the discrepancy between SLAVE and LPJ logic of allocation, thus leads to excess heartwood
          ! and thus carbon accumulation!
          ! should be removed.

          IF ( tree(j) ) THEN

!!$             sm2(:) = 0.0
!!$             WHERE ( d_ind(:,j) .GT. 0.0 ) 
!!$
!!$                ! ratio of above / total sap parts
!!$                sm_at(:) = biomass(:,j,isapabove) / &
!!$                     ( biomass(:,j,isapabove) + biomass(:,j,isapbelow) )
!!$
!!$                ! woodmass of an individual
!!$
!!$                woodmass(:) = &
!!$                     ( biomass(:,j,isapabove) + biomass(:,j,isapbelow) + &
!!$                     biomass(:,j,iheartabove) + biomass(:,j,iheartbelow) ) / ind(:,j)
!!$
!!$                ! crown area (m**2) depends on stem diameter (pipe model)
!!$                dia(:) = ( woodmass(:) / ( pipe_density * pi/4. * pipe_tune2 ) ) &
!!$                     ** ( 1. / ( 2. + pipe_tune3 ) )
!!$
!!$                b1(:) = pipe_k1 / ( sla(j) * pipe_density*pipe_tune2 * dia(:)**pipe_tune3 ) * &
!!$                     ind(:,j)
!!$                sm2(:) = lm_lastyearmax(:,j) / b1(:)
!!$
!!$             ENDWHERE

             sm2(:)=biomass(:,j,isapabove) + biomass(:,j,isapbelow)

             WHERE ( ( d_ind(:,j) .GT. min_stomate ) .AND. &
                  ( biomass(:,j,isapabove) + biomass(:,j,isapbelow) ) .GT. sm2(:) )

                biomass(:,j,iheartabove) = biomass(:,j,iheartabove) + &
                     ( biomass(:,j,isapabove) - sm2(:) * sm_at(:) )
                biomass(:,j,isapabove) = sm2(:) * sm_at(:)

                biomass(:,j,iheartbelow) = biomass(:,j,iheartbelow) + &
                     ( biomass(:,j,isapbelow) - sm2(:) * (un - sm_at) )
                biomass(:,j,isapbelow) = sm2(:) * (un - sm_at(:))

             ENDWHERE

          ENDIF        ! tree

       ENDIF          ! natural

    ENDDO            ! loop over pfts

    !
    ! 5 history
    !

    d_ind = d_ind / dt

    CALL histwrite (hist_id_stomate, 'IND_ESTAB', itime, d_ind, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'ESTABTREE', itime, estab_rate_max_tree, npts, hori_index)
    CALL histwrite (hist_id_stomate, 'ESTABGRASS', itime, estab_rate_max_grass, npts, hori_index)

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving establish'

  END SUBROUTINE establish

END MODULE lpj_establish
